import java.lang.Math;

/**
   <strong>Clase que implementa la representacion de vectores bi-dimensionales.
   Contiene diversos metodos para la manipulacion de vectores.</strong>
   @author Eduardo Gonz&aacute;lez Fisher <br>ROL: 2021010-9
   @version 0.9
   @since 1.5
   <p><i>Este c&oacute;digo ha sido probadon con J2SE 1.5. El autor de este c&oacute;digo no garantiza su correcto funcionanmiento en versiones anteriores.</i></p>
*/
public class Vector
{
    /**
      Contiene la coordenada X en el sistema cartesiano.
    */
    private double x;
    
    /**
       Contiene la coordenada Y en el sisteman cartesiano.
    */
    private double y;
 
    /**
       Angulo del sistema de coordenadas polares, en radianes.
    */
    private double ang;

    /**
       Radio del sistema polar.
    */
    private float radio;
    
    /**
       Constructor por omision.
       <br>Construye un vector con las siguientes caracteristicas:
       <ol><li>X = 4</li>
       <li>Y = 3</li>
       <li>Radio = 5</li>
       <li>Angulo = 0.6435011087932844</li></ol>
    */
    Vector() {
	x = 4;
	y = 3;
	setPolares(x,y);
    }


    /**
       Constructor Cartesiano
       @param a coordenada X
       @param b coordenada Y
    */
    Vector(double a, double b) {
	x = a;
	y = b;
	setPolares(a,b);
    }

    /**
       Constructor Polar
       @param r radio
       @param alfa angulo
    */
    Vector(float r, double alfa) {
	radio = r;
	ang = alfa;
	setCartesianas(r,alfa);
    }

    /**
       Metodo que traslada el vector.
       @param v Vector a agregar.
       <p>Este metodo consiste en sumarle al vector otro vector.
       De esta forma el vector original pasa a ser el resultado de la suma
       de otro vector con si mismo</p>
    */
    public void sumarle(Vector v) {
	this.x = this.x + v.x;
	this.y = this.y + v.y;
	setPolares(this.x,this.y);
    }

    /**
       Metodo para multiplicar un vector.
       @param n Factor a multiplicar.
       <p>Con este metodo es posible modificar el radio de un vector
       multiplicandolo por un factor. Esto no modifica el angulo del vector</p>
    */
    public void multiplicacionPorEscalar(float n) {
	this.radio *= n;
	setCartesianas(this.radio, this.ang);
    }

    /**
       Metodo para rotar un vector.
       @param g Angulo a rotar.
       <p>Con este metodo es posible modificar el angulo de un vector
       haciendo girar el vector en un numero determinado de radianes</p>
    */
    public void rotar(double g) {
	this.ang += g;
	setCartesianas(this.radio, this.ang);
    }

    /**
       Metodo para ajustar las coordenadas cartesianas.
       @param r Radio
       @param alfa Angulo
       <p>Este metodo permite a partir de coordenadas polares, determinar
       las coordenadas cartesianas de un vector.</p>
    */
    private void setCartesianas(float r, double alfa) {
	this.x = r * Math.cos(alfa);
	this.y = r * Math.sin(alfa);
    }

    /**
       Metodo para ajustar las coordenadas polares.
       @param a coordenada X
       @param b coorednada Y
       <p>Este metodo permite a partir de coordenados cartesianas determinar
       las coordenadas polares de un vector.</p>
    */
    private void setPolares (double a, double b) {
	this.radio = (float)Math.sqrt(a*a+b*b);
	if (a<0) 
	    this.ang = Math.PI+Math.atan(b/a);
	else
	    this.ang = Math.atan(b/a);
    }

    /**
       Metodo que hace que el vector se pueda interpretar en texto legible.
       @return Cadena de texto con los atributos del vector.
    */
        
    public String toString() {
	return "Radio: " + radio + "  Angulo: " + ang
	    + "\t Coord.X: " + x + "  Coord.Y: " + y;
    }

    /**
       Verifica si dos vectores son iguales.
       @param v Vector para comparar
       @return TRUE si son iguales, FALSE en caso contrario
       <p>Debido a errores que se pueden producir al manipular ambos sistemas de coordenadas,
       se comparan las coordenadas cartesianas y las polares. Si alguna de las dos coinciden 
       se consideran iguales los vectores.</p>
    */
    public boolean equals(Vector v) {
	if (this.x == v.x && this.y == v.y)
	    return true;
	if (this.radio == v.radio && this.ang == v.ang)
	    return true;
	else
	    return false;
    }

    /**
       Calcula el producto punto entre dos vectores.
       @param a Vector 1
       @param b Vector 2
       @return Producto punto, en numero flotante.
    */
    public static float productoPunto(Vector a , Vector b) {
	return (float)(a.radio*b.radio*Math.cos(a.ang-b.ang));
    }

    /**
       Suma dos vectores.
       @param a Vector 1
       @param b Vector 2
       @return Entrega un vector con el resultado.
    */
    public static Vector suma (Vector a, Vector b) {
	Vector q = new Vector();
	q.x = a.x + b.x;
	q.y = a.y + b.y;
	q.setPolares(q.x,q.y);
	return q;
    }

    /**
       Resta dos vectores.
       @param a Vector 1
       @param b Vector 2
       @return Entrega un vector con el resultado.
    */
    public static Vector resta (Vector a, Vector b) {
	Vector q = new Vector();
	q.x = a.x - b.x;
	q.y = a.y - b.y;
	q.setPolares(q.x,q.y);
	return q;
    }



}
